///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2014 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Mar 20, 2014	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.graph;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.data.TdbGraphRecord;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.graph.TdbGraph;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * Shows how to add an edge to a TRIP graph database
 * 
 * THIS CLASS USES EXPERIMENTAL FEATURES. USE IT WITH CAUTION!
 */
public class CreateEdge 
{
    private TdbSession m_session;
    
    public CreateEdge()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** CreateEdge TRIP graph example ****", false);
    }
    
    public void close()
    {
        if (m_session != null)
        {
            
            try
            {
                m_session.logout();
            }
            catch (TdbException ignore)
            {
            }
            finally
            {
                m_session = null;
            }
        }
    }


   // Show any success message that might be forthcoming from the server

   private static void showMessages(TdbMessageProvider prv)
   {
      for (String msg : prv.messageList())
      {
         System.out.println(msg);
      }
      prv.resetMessages();
   }
   

    private String readnoblank(Console stdin, String prefix, String defaultValue)
           throws TdbException
   {
       String inval;
       do
       {
           inval = stdin.readln(prefix,defaultValue,true);
       }
       while (inval.length()==0);
       return inval;
   }
   
   
   public void process(Console stdin) throws TdbException
   {
       TdbGraph graph;
       TdbRecord source, target;
       String graphDbName;
       String sourceDbName;
       String targetDbName;
       String edgeName;
       int sourceRid;
       int targetRid;
       double weight = 0.00;
       boolean autocommit = true;
       
       // The graph database to use
       graphDbName = readnoblank(stdin,"Graph database","GRAPHBASE");
       graph = new TdbGraph(m_session, graphDbName);
       
       // Database for source record 
       sourceDbName = readnoblank(stdin,"Database for source record","");       
       source = new TdbRecord(m_session,sourceDbName,false);
       
       // ID of source record 
       do
       {
           sourceRid = stdin.readint("Source record ID",0,true);
       } while (sourceRid<1);
       
       source.setRecordId(sourceRid);
       source.get();

       // The name of the edge
       edgeName = readnoblank(stdin,"Edge name (relation)","");
       
       // Database for target record
       targetDbName = readnoblank(stdin,"Database for target record","");       
       target = new TdbRecord(m_session, targetDbName, false);
       
       // ID of target record
       do
       {
           targetRid = stdin.readint("Target record ID",0,true);
       } while (targetRid<1);
       
       target.setRecordId(targetRid);
       target.get();
       
       if (stdin.readbool("Assign a weight?",false,true))
       {
           weight = Double.parseDouble(readnoblank(stdin,"Weight (floating point value)","0.00"));
           autocommit = false;
       }

       TdbGraphRecord gr = graph.createEdge(edgeName, source, target, autocommit);
       if (!autocommit)
       {
           gr.setWeight(weight);
           gr.commit();
       }

       showMessages(graph);       
       System.out.println();
       System.out.println("Created graph edge \""+gr.getName()+"\".");
       System.out.println("     Record ID = "+gr.getRecordId());
       System.out.println("   Record Name = "+gr.getRecordName());
       
       graph.close();
   }
   
   
   public static void main(String[] args)
   {
       CreateEdge app = null;
       Console stdin = new Console();
       try
       {
           app = new CreateEdge();
           app.process(stdin);
       }
       catch (TdbException tx)
       {
           System.err.println(tx.getDebugMessage());
       }
       catch (Exception ox)
       {
           ox.printStackTrace(System.err);
       }
       finally
       {
           if (app != null)
               app.close();
       }
   }
    
}
