///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2014 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Feb 21, 2014	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.graph;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.graph.TdbGraph;
import com.tietoenator.trip.jxp.graph.TdbGraphPath;
import com.tietoenator.trip.jxp.graph.TdbGraphPathEdge;
import com.tietoenator.trip.jxp.graph.TdbGraphPathVertex;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.text.DecimalFormat;

/**
 * Example of how to use obtain the shortest path in a graph using the graph 
 * classes in TRIPjxp.
 *
 * THIS CLASS USES EXPERIMENTAL FEATURES. USE IT WITH CAUTION!
 */
public class ShortestPath 
{

    private TdbSession m_session;

    public ShortestPath()
            throws TdbException 
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** ShortestPath TRIP graph example ****", false);
    }

    public void close() 
    {
        if (m_session != null) 
        {
            try 
            {
                m_session.logout();
            } 
            catch (TdbException ignore) 
            {
            } 
            finally 
            {
                m_session = null;
            }
        }
    }

    /**
     * Show any success message that might be forthcoming from the server
     *
     * @param prv Message provider subclass
     */
    private static void showMessages(TdbMessageProvider prv) 
    {
        for (String msg : prv.messageList()) 
        {
            System.out.println(msg);
        }
        prv.resetMessages();
    }
    
    private static void showPath(TdbGraphPath path)
    {
        TdbGraphPathVertex vtx;
        TdbGraphPathEdge edge;
        DecimalFormat df = new DecimalFormat("#.##");

        // Show result
        vtx = path.getStartVertex();
        while (vtx != null)
        {
            System.out.print(vtx.getDatabaseName());
            System.out.print(".");
            System.out.print(vtx.getRecordId());

            edge = vtx.getOutputEdge();
            if (edge != null)
            {
                System.out.print(" --(");
                System.out.print(edge.getName());
                System.out.print(":");
                System.out.print(df.format(edge.getWeight()));
                System.out.print(")--> ");

                vtx = edge.getTarget();
            }
            else
            {
                vtx = null;
            }
        }
        System.out.println();
    }

    private String readnoblank(Console stdin, String prefix, String defaultValue)
            throws TdbException 
    {
        String inval;
        do 
        {
            inval = stdin.readln(prefix, defaultValue, true);
        } 
        while (inval.length() == 0);
        return inval;

    }

    public void process(Console stdin) throws TdbException 
    {
        TdbGraph graph;
        String graphDbName;
        String sourceDbName;
        String targetDbName;
        String edgeName;
        int sourceRid;
        int sourceSet = 0;
        int targetRid;
        int targetSet = 0;
        String[] edgeNames = null;
        
        TdbSearch normalSearch = new TdbSearch(m_session);
        // normalSearch.execute("DEFINE FIND MIN=0");
        
        // Database for start record
        sourceDbName = readnoblank(stdin, "Database for start record", "ALICE");
        normalSearch.execute("BASE " + sourceDbName);

        // ID of source record to explore relations from
        sourceRid = stdin.readint("Start record ID", 1, true);
        normalSearch.execute("FIND R=" + sourceRid);
        sourceSet = normalSearch.getLastSearchSet().getSearchId();
        
        // Database for target record
        targetDbName = readnoblank(stdin, "Database for target record", "ALICE");
        normalSearch.execute("BASE " + targetDbName);

        // ID of target record
        targetRid = stdin.readint("Target record ID", 5, true);
        normalSearch.execute("FIND R=" + targetRid);
        targetSet = normalSearch.getLastSearchSet().getSearchId();
        
        // The graph database to use
        graphDbName = readnoblank(stdin, "Graph database", "GRAPHBASE");
        graph = new TdbGraph(m_session, graphDbName);
        
        // The name of the relation
        edgeName = readnoblank(stdin, "Edge name (relation)", "FRIEND");
        edgeNames = new String[1];
        edgeNames[0] = edgeName;

        TdbGraphPath path = graph.getGraphSet().shortestPath(sourceSet, edgeNames, targetSet);
        showMessages(graph);
        showPath(path);
 
        normalSearch.close();
        graph.close();
    }

    public static void main(String[] args) 
    {
        ShortestPath app = null;
        Console stdin = new Console();
        try 
        {
            app = new ShortestPath();
            app.process(stdin);
        }
        catch (TdbException tx) 
        {
            System.err.println(tx.getDebugMessage());
        }
        catch (Exception ox) 
        {
            ox.printStackTrace(System.err);
        }
        finally 
        {
            if (app != null) 
            {
                app.close();
            }
        }
    }
}
