///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp programming examples
//
// Copyright (C) 2006-2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//    GF Jul 17, 2006   Created
//    TF Jul 13, 2008   Added support for TdbLocalSession 
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.util;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.session.TdbGridSession;
import com.tietoenator.trip.jxp.session.TdbSession;
import com.tietoenator.trip.jxp.session.TdbTripNetSession;
import com.tietoenator.trip.jxp.session.TdbLocalSession;
import com.tietoenator.trip.jxp.session.TdbWebSession;

/**
 * Factory class for creating all kinds of TRIPjxp sessions, whether via TRIPnet,
 * TRIP web service or TRIPgrid. Depending on the type of session created, the
 * user's credentials are used to either login or to authenticate.
 */
public class SessionFactory
{
   // disable construction access to this class

   private SessionFactory()
   {
   }

   /**
    * Utility method to create a session on a remote server via either TRIPnet
    * or using the TRIP web service in grid or standalone mode
    *
    * @param prompt A text prompt to be shown on screen before any other stuff
    * @param allowGrid Set true if the session can address a grid, false otherwise
    * @return A usable TdbSession instance
    * @throws TdbException
    */
   public static TdbSession newInstance(String prompt, boolean allowGrid)
           throws TdbException
   {
      TdbSession session = null;
      Console stdin = new Console();
      String type, host, username = null, password = null;
      char ct;
      int dfltport, port;
      boolean encrypted = false;

      System.out.println(prompt + "\n");
      for (;;)
      {
         if (allowGrid)
         {
            type = stdin.readln("Session type (net, local, web, grid)", "net");
            if (type == null)
            {
               return null;
            }
            ct = type.charAt(0);
            if (ct != 'n' && ct != 'w' && ct != 'g' && ct != 'l')
            {
               continue;
            }
         }
         else
         {
            type = stdin.readln("Session type (net, local, web)", "net");
            if (type == null)
            {
               return null;
            }
            ct = type.charAt(0);
            if (ct != 'n' && ct != 'w' && ct != 'l')
            {
               continue;
            }
         }

         if (ct == 'n')
         {
            dfltport = 23457;
         }
         else
         {
            dfltport = 80;
         }

         if (ct != 'l')
         {
            host = stdin.readln("Server", "localhost");
            port = stdin.readint("Port#", dfltport, true);
         }
         else
         {
            host = "localhost";
            port = dfltport;
         }
         
         if (ct == 'n')
         {
             encrypted = stdin.readbool("Encrypt the session?", false, true);
         }

         // Connect to the server using the appropriate protocol
         switch (ct)
         {
         case 'g':
            session = new TdbGridSession(host, port);
            break;
         case 'n':
            session = new TdbTripNetSession(host, port, encrypted);
            break;
         case 'w':
            session = new TdbWebSession(host, port);
            break;
         case 'l':
            session = new TdbLocalSession();
            break;
         }
         break;
      }

      // Collect username / password, login to the server
      while (username == null)
      {
         username = stdin.readln("Username", null);
      }

      while (password == null)
      {
         password = stdin.readln("Password", null);
      }

      if (ct == 'g')
      {
         session.authenticate(username, password);
      }
      else
      {
         session.login(username, password);
      }

      return session;
   }
}
